@@ -34,6 +34,7 @@
 #include <KoColor.h>
 #include "kis_canvas_resource_provider.h"
 #include <KisHandlePainterHelper.h>
+#include "KoPathPointTypeCommand.h"
 
 #include <klocalizedstring.h>
 
@@ -129,7 +130,10 @@ void KoCreatePathTool::mousePressEvent(KoPointerEvent *event)
 
     const bool isOverFirstPoint = d->shape &&
                                   handleGrabRect(d->firstPoint->point()).contains(event->point);
-    bool haveCloseModifier = (listeningToModifiers() && (event->modifiers() & Qt::ShiftModifier));
+
+    const bool haveCloseModifier = d->enableClosePathShortcut &&
+        d->shape && d->shape->pointCount() > 2 &&
+        (event->modifiers() & Qt::ShiftModifier);
 
     if ((event->button() == Qt::LeftButton) && haveCloseModifier && !isOverFirstPoint) {
         endPathWithoutLastPoint();
@@ -212,12 +216,6 @@ void KoCreatePathTool::mousePressEvent(KoPointerEvent *event)
         d->angleSnapStrategy->setStartPoint(d->activePoint->point());
 }
 
-bool KoCreatePathTool::listeningToModifiers()
-{
-    Q_D(KoCreatePathTool);
-    return d->listeningToModifiers;
-}
-
 bool KoCreatePathTool::pathStarted()
 {
     Q_D(KoCreatePathTool);
@@ -229,6 +227,12 @@ bool KoCreatePathTool::tryMergeInPathShape(KoPathShape *pathShape)
     return addPathShapeImpl(pathShape, true);
 }
 
+void KoCreatePathTool::setEnableClosePathShortcut(bool value)
+{
+    Q_D(KoCreatePathTool);
+    d->enableClosePathShortcut = value;
+}
+
 void KoCreatePathTool::mouseDoubleClickEvent(KoPointerEvent *event)
 {
     //remove handle
@@ -290,6 +294,38 @@ void KoCreatePathTool::mouseMoveEvent(KoPointerEvent *event)
         }
     } else {
         d->activePoint->setPoint(snappedPosition);
+
+        if (!d->prevPointWasDragged && d->autoSmoothCurves) {
+            KoPathPointIndex index = d->shape->pathPointIndex(d->activePoint);
+            if (index.second > 0) {
+
+                KoPathPointIndex prevIndex(index.first, index.second - 1);
+                KoPathPoint *prevPoint = d->shape->pointByIndex(prevIndex);
+
+                if (prevPoint) {
+                    KoPathPoint *prevPrevPoint = 0;
+
+                    if (index.second > 1) {
+                        KoPathPointIndex prevPrevIndex(index.first, index.second - 2);
+                        prevPrevPoint = d->shape->pointByIndex(prevPrevIndex);
+                    }
+
+                    if (prevPrevPoint) {
+                        const QPointF control1 = prevPoint->point() + 0.3 * (prevPrevPoint->point() - prevPoint->point());
+                        prevPoint->setControlPoint1(control1);
+                    }
+
+                    const QPointF control2 = prevPoint->point() + 0.3 * (d->activePoint->point() - prevPoint->point());
+                    prevPoint->setControlPoint2(control2);
+
+                    const QPointF activeControl = d->activePoint->point() + 0.3 * (prevPoint->point() - d->activePoint->point());
+                    d->activePoint->setControlPoint1(activeControl);
+
+                    KoPathPointTypeCommand::makeCubicPointSmooth(prevPoint);
+                }
+            }
+        }
+
     }
 
     canvas()->updateCanvas(d->shape->boundingRect());
@@ -302,8 +338,8 @@ void KoCreatePathTool::mouseReleaseEvent(KoPointerEvent *event)
 
     if (! d->shape || (event->buttons() & Qt::RightButton)) return;
 
-    d->listeningToModifiers = true; // After the first press-and-release
     d->repaintActivePoint();
+    d->prevPointWasDragged  = d->pointIsDragged;
     d->pointIsDragged = false;
     KoPathPoint *lastActivePoint = d->activePoint;
 
@@ -325,6 +361,11 @@ void KoCreatePathTool::mouseReleaseEvent(KoPointerEvent *event)
         d->firstPoint->setControlPoint1(d->activePoint->controlPoint1());
         delete d->shape->removePoint(d->shape->pathPointIndex(d->activePoint));
         d->activePoint = d->firstPoint;
+
+        if (!d->prevPointWasDragged && d->autoSmoothCurves) {
+            KoPathPointTypeCommand::makeCubicPointSmooth(d->activePoint);
+        }
+
         d->shape->closeMerge();
 
         // we are closing the path, so reset the existing start path point
@@ -408,6 +449,7 @@ void KoCreatePathTool::activate(ToolActivation activation, const QSet<KoShape*>
 
     // retrieve the actual global handle radius
     d->handleRadius = handleRadius();
+    d->loadAutoSmoothValueFromConfig();
 
     // reset snap guide
     canvas()->updateCanvas(canvas()->snapGuide()->boundingRect());
@@ -502,6 +544,14 @@ QList<QPointer<QWidget> > KoCreatePathTool::createOptionWidgets()
 
     QList<QPointer<QWidget> > list;
 
+    QCheckBox *smoothCurves = new QCheckBox(i18n("Autosmooth curve"));
+    smoothCurves->setObjectName("smooth-curves-widget");
+    smoothCurves->setChecked(d->autoSmoothCurves);
+    connect(smoothCurves, SIGNAL(toggled(bool)), this, SLOT(autoSmoothCurvesChanged(bool)));
+    connect(this, SIGNAL(sigUpdateAutoSmoothCurvesGUI(bool)), smoothCurves, SLOT(setChecked(bool)));
+
+    list.append(smoothCurves);
+
     QWidget *angleWidget = new QWidget();
     angleWidget->setObjectName("Angle Constraints");
     QGridLayout *layout = new QGridLayout(angleWidget);
