@@ -747,7 +747,6 @@ void KoShape::update() const
 
 void KoShape::updateAbsolute(const QRectF &rect) const
 {
-
     if (rect.isEmpty() && !rect.isNull()) {
         return;
     }
@@ -755,7 +754,7 @@ void KoShape::updateAbsolute(const QRectF &rect) const
     Q_D(const KoShape);
 
     if (!d->shapeManagers.empty() && isVisible()) {
-        Q_FOREACH (KoShapeManager * manager, d->shapeManagers) {
+        Q_FOREACH (KoShapeManager *manager, d->shapeManagers) {
             manager->update(rect);
         }
     }
@@ -777,7 +776,6 @@ QRectF KoShape::outlineRect() const
 
 QPainterPath KoShape::shadowOutline() const
 {
-    Q_D(const KoShape);
     if (background()) {
         return outline();
     }
@@ -2248,7 +2246,6 @@ void KoShape::saveOdfCommonChildElements(KoShapeSavingContext &context) const
                 escapeDirection = "down";
                 break;
             default:
-                // fall through
                 break;
             }
             if(!escapeDirection.isEmpty()) {
@@ -2284,7 +2281,6 @@ void KoShape::saveOdfCommonChildElements(KoShapeSavingContext &context) const
                 alignment = "bottom-right";
                 break;
             default:
-                // fall through
                 break;
             }
             if(!alignment.isEmpty()) {
