@@ -84,10 +84,13 @@ public:
             return true;
         }
 
-        KisPaintDeviceSP originalDevice = layer->original();
-        originalDevice->clear(m_updateRect);
+        const QRect originalUpdateRect =
+            layer->projectionPlane()->needRectForOriginal(m_updateRect);
 
-        const QRect applyRect = m_updateRect & m_projection->extent();
+        KisPaintDeviceSP originalDevice = layer->original();
+        originalDevice->clear(originalUpdateRect);
+
+        const QRect applyRect = originalUpdateRect & m_projection->extent();
 
         // If the intersection of the updaterect and the projection extent is
         //      null, we are finish here.
@@ -216,13 +219,24 @@ void KisAsyncMerger::startMerge(KisBaseRectsWalker &walker, bool notifyClones) {
         KisMergeWalker::JobItem item = leafStack.pop();
         KisProjectionLeafSP currentLeaf = item.m_leaf;
 
-        if(currentLeaf->isRoot()) continue;
+        /**
+         * In some unidentified cases teh nodes might be removed
+         * while the updates are still running. We have no proof
+         * of it yet, so just add a safety assert here.
+         */
+        KIS_SAFE_ASSERT_RECOVER_RETURN(currentLeaf);
+        KIS_SAFE_ASSERT_RECOVER_RETURN(currentLeaf->node());
 
         // All the masks should be filtered by the walkers
-        Q_ASSERT(currentLeaf->isLayer());
+        KIS_SAFE_ASSERT_RECOVER_RETURN(currentLeaf->isLayer());
 
         QRect applyRect = item.m_applyRect;
 
+        if (currentLeaf->isRoot()) {
+            currentLeaf->projectionPlane()->recalculate(applyRect, walker.startNode());
+            continue;
+        }
+
         if(item.m_position & KisMergeWalker::N_EXTRA) {
             // The type of layers that will not go to projection.
 
@@ -237,8 +251,9 @@ void KisAsyncMerger::startMerge(KisBaseRectsWalker &walker, bool notifyClones) {
         }
 
 
-        if(!m_currentProjection)
+        if (!m_currentProjection) {
             setupProjection(currentLeaf, applyRect, useTempProjections);
+        }
 
         KisUpdateOriginalVisitor originalVisitor(applyRect,
                                                  m_currentProjection,
