@@ -25,18 +25,7 @@
 #include "kis_default_bounds.h"
 #include "kis_image.h"
 
-enum SelectionMode {
-    PIXEL_SELECTION,
-    SHAPE_PROTECTION
-};
-
-enum SelectionAction {
-    SELECTION_REPLACE,
-    SELECTION_ADD,
-    SELECTION_SUBTRACT,
-    SELECTION_INTERSECT,
-    SELECTION_DEFAULT
-};
+#include "KisSelectionTags.h"
 
 #include "kis_pixel_selection.h"
 
@@ -83,6 +72,11 @@ public:
      */
     virtual ~KisSelection();
 
+    /**
+     * Create a new selection using the content of copySource as the mask.
+     */
+    KisSelection(const KisPaintDeviceSP copySource, KritaUtils::DeviceCopyMode copyMode, KisDefaultBoundsBaseSP defaultBounds);
+
     /**
      * The paint device of the pixel selection should report
      * about it's setDirty events to its parent. The creator
@@ -198,15 +192,27 @@ public:
 
     void notifySelectionChanged();
 
+    /**
+     * Request rerendering of the shape selection component in a
+     * compressed way. Usually, you don't need to call it manually,
+     * because all the work is done by KisShapeSelectionModel.
+     */
+    void requestCompressedProjectionUpdate(const QRect &rc);
+
     /// XXX: This method was marked KDE_DEPRECATED but without information on what to
     /// replace it with. Undeprecate, therefore.
     quint8 selected(qint32 x, qint32 y) const;
 
+    KisNodeWSP parentNode() const;
+
 private:
     friend class KisSelectionTest;
     friend class KisMaskTest;
     friend class KisAdjustmentLayerTest;
-    KisNodeWSP parentNode() const;
+    friend class KisUpdateSelectionJob;
+    friend class KisSelectionUpdateCompressor;
+    friend class KisDeselectActiveSelectionCommand;
+
 
     void copyFrom(const KisSelection &rhs);
 
