@@ -22,6 +22,8 @@
 
 #include <QList>
 #include "kis_single_action_shortcut.h"
+#include "KisInputActionGroup.h"
+#include <functional>
 
 class QEvent;
 class QWheelEvent;
@@ -34,6 +36,7 @@ class KisStrokeShortcut;
 class KisTouchShortcut;
 class KisNativeGestureShortcut;
 
+
 /**
  * The class that manages connections between shortcuts and actions.
  *
@@ -199,6 +202,16 @@ public:
      */
     void reinitialize();
 
+    /**
+     * Resets the internal state of the matcher, tries to resync it to the state
+     * passed via argument and activates the prepared action if possible.
+     *
+     * This synchronization should happen when the user hovers Krita windows,
+     * **without** having keyboard focus set to it (therefore matcher cannot
+     * get key press and release events).
+     */
+    void recoveryModifiersWithoutFocus(const QVector<Qt::Key> &keys);
+
     /**
      * Kirta lost focus, it means that all the running actions should be ended
      * forcefully.
@@ -219,6 +232,8 @@ public:
      */
     void clearShortcuts();
 
+    void setInputActionGroupsMaskCallback(std::function<KisInputActionGroupsMask()> func);
+
 private:
     friend class KisInputManagerTest;
 
