@@ -56,7 +56,6 @@
 #include "kis_convolution_painter.h"
 #include "kis_convolution_kernel.h"
 #include "kis_debug.h"
-#include "KisDocument.h"
 #include "kis_fill_painter.h"
 #include "kis_group_layer.h"
 #include "kis_layer.h"
@@ -196,9 +195,15 @@ void KisSelectionManager::setup(KisActionManager* actionManager)
     m_imageResizeToSelection  = actionManager->createAction("resizeimagetoselection");
     connect(m_imageResizeToSelection, SIGNAL(triggered()), this, SLOT(imageResizeToSelection()));
 
+    action = actionManager->createAction("edit_selection");
+    connect(action, SIGNAL(triggered()), SLOT(editSelection()));
+
     action = actionManager->createAction("convert_to_vector_selection");
     connect(action, SIGNAL(triggered()), SLOT(convertToVectorSelection()));
 
+    action = actionManager->createAction("convert_to_raster_selection");
+    connect(action, SIGNAL(triggered()), SLOT(convertToRasterSelection()));
+
     action = actionManager->createAction("convert_shapes_to_vector_selection");
     connect(action, SIGNAL(triggered()), SLOT(convertShapesToVectorSelection()));
 
@@ -219,7 +224,7 @@ void KisSelectionManager::setup(KisActionManager* actionManager)
 void KisSelectionManager::setView(QPointer<KisView>imageView)
 {
     if (m_imageView && m_imageView->canvasBase()) {
-        disconnect(m_imageView->canvasBase()->toolProxy(), SIGNAL(toolChanged(const QString&)), this, SLOT(clipboardDataChanged()));
+        disconnect(m_imageView->canvasBase()->toolProxy(), SIGNAL(toolChanged(QString)), this, SLOT(clipboardDataChanged()));
 
         KoSelection *selection = m_imageView->canvasBase()->globalShapeManager()->selection();
         selection->disconnect(this, SLOT(shapeSelectionChanged()));
@@ -233,7 +238,7 @@ void KisSelectionManager::setView(QPointer<KisView>imageView)
 
     m_imageView = imageView;
     if (m_imageView) {
-        connect(m_imageView->canvasBase()->selectedShapesProxy(), SIGNAL(selectionChanged()), this, SLOT(shapeSelectionChanged()));
+        connect(m_imageView->canvasBase()->selectedShapesProxy(), SIGNAL(selectionChanged()), this, SLOT(shapeSelectionChanged()), Qt::UniqueConnection);
 
         KisSelectionDecoration* decoration = qobject_cast<KisSelectionDecoration*>(m_imageView->canvasBase()->decoration("selection").data());
         if (!decoration) {
@@ -244,7 +249,7 @@ void KisSelectionManager::setView(QPointer<KisView>imageView)
         m_selectionDecoration = decoration;
         connect(this, SIGNAL(currentSelectionChanged()), decoration, SLOT(selectionChanged()));
         connect(m_imageView->image()->undoAdapter(), SIGNAL(selectionChanged()), SLOT(selectionChanged()));
-        connect(m_imageView->canvasBase()->toolProxy(), SIGNAL(toolChanged(const QString&)), SLOT(clipboardDataChanged()));
+        connect(m_imageView->canvasBase()->toolProxy(), SIGNAL(toolChanged(QString)), SLOT(clipboardDataChanged()));
 
     }
 }
@@ -280,13 +285,22 @@ bool KisSelectionManager::haveShapesInClipboard()
     return paste.hasShapes();
 }
 
-bool KisSelectionManager::havePixelSelectionWithPixels()
+bool KisSelectionManager::haveAnySelectionWithPixels()
 {
     KisSelectionSP selection = m_view->selection();
-    if (selection && selection->hasPixelSelection()) {
-        return !selection->pixelSelection()->selectedRect().isEmpty();
-    }
-    return false;
+    return selection && selection->hasPixelSelection();
+}
+
+bool KisSelectionManager::haveShapeSelectionWithShapes()
+{
+    KisSelectionSP selection = m_view->selection();
+    return selection && selection->hasShapeSelection();
+}
+
+bool KisSelectionManager::haveRasterSelectionWithPixels()
+{
+    KisSelectionSP selection = m_view->selection();
+    return selection && selection->hasPixelSelection() && !selection->hasShapeSelection();
 }
 
 void KisSelectionManager::updateGUI()
@@ -410,12 +424,62 @@ void KisSelectionManager::reselect()
     factory.run(m_view);
 }
 
+
+#include <KoToolManager.h>
+#include <KoInteractionTool.h>
+
+void KisSelectionManager::editSelection()
+{
+    KisSelectionSP selection = m_view->selection();
+    if (!selection) return;
+
+    KisAction *action = m_view->actionManager()->actionByName("show-global-selection-mask");
+    KIS_SAFE_ASSERT_RECOVER_RETURN(action);
+
+    if (!action->isChecked()) {
+        action->setChecked(true);
+        emit action->toggled(true);
+        emit action->triggered(true);
+    }
+
+    KisNodeSP node = selection->parentNode();
+    KIS_SAFE_ASSERT_RECOVER_RETURN(node);
+
+    m_view->nodeManager()->slotNonUiActivatedNode(node);
+
+    if (selection->hasShapeSelection()) {
+        KisShapeSelection *shapeSelection = dynamic_cast<KisShapeSelection*>(selection->shapeSelection());
+        KIS_SAFE_ASSERT_RECOVER_RETURN(shapeSelection);
+
+        KoToolManager::instance()->switchToolRequested(KoInteractionTool_ID);
+
+        QList<KoShape*> shapes = shapeSelection->shapes();
+
+        if (shapes.isEmpty()) {
+            KIS_SAFE_ASSERT_RECOVER_NOOP(0 && "no shapes");
+            return;
+        }
+
+        Q_FOREACH (KoShape *shape, shapes) {
+            m_view->canvasBase()->selectedShapesProxy()->selection()->select(shape);
+        }
+    } else {
+        KoToolManager::instance()->switchToolRequested("KisToolTransform");
+    }
+}
+
 void KisSelectionManager::convertToVectorSelection()
 {
     KisSelectionToVectorActionFactory factory;
     factory.run(m_view);
 }
 
+void KisSelectionManager::convertToRasterSelection()
+{
+    KisSelectionToRasterActionFactory factory;
+    factory.run(m_view);
+}
+
 void KisSelectionManager::convertShapesToVectorSelection()
 {
     KisShapesToVectorSelectionActionFactory factory;
@@ -544,7 +608,7 @@ void KisSelectionManager::paintSelectedShapes()
     KisFigurePaintingToolHelper helper(actionName,
                                        image,
                                        paintLayer.data(),
-                                       m_view->resourceProvider()->resourceManager(),
+                                       m_view->canvasResourceProvider()->resourceManager(),
                                        KisPainter::StrokeStyleBrush,
                                        KisPainter::FillStyleNone);
 
@@ -583,7 +647,7 @@ void KisSelectionManager::slotStrokeSelection()
 
         return;
     }
-    KisNodeSP currentNode = m_view->resourceProvider()->resourceManager()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeWSP>();
+    KisNodeSP currentNode = m_view->canvasResourceProvider()->resourceManager()->resource(KisCanvasResourceProvider::CurrentKritaNode).value<KisNodeWSP>();
     bool isVectorLayer = false;
     if (currentNode->inherits("KisShapeLayer")) {
         isVectorLayer = true;
@@ -606,3 +670,77 @@ void KisSelectionManager::slotStrokeSelection()
 
 
 }
+
+#include "kis_image_barrier_locker.h"
+#include "kis_selection_tool_helper.h"
+
+void KisSelectionManager::selectOpaqueOnNode(KisNodeSP node, SelectionAction action)
+{
+    KisImageSP image = m_view->image();
+
+    if (!m_view->blockUntilOperationsFinished(image)) {
+        return;
+    }
+
+    KUndo2MagicString actionName;
+    KisPixelSelectionSP tmpSel = KisPixelSelectionSP(new KisPixelSelection());
+    KisCanvas2 *canvas = m_view->canvasBase();
+
+
+    {
+        KisImageBarrierLocker locker(image);
+
+        KisPaintDeviceSP device = node->projection();
+        if (!device) device = node->paintDevice();
+        if (!device) device = node->original();
+        KIS_ASSERT_RECOVER_RETURN(canvas && device);
+
+        QRect rc = device->exactBounds();
+        if (rc.isEmpty()) return;
+
+        /**
+         * If there is nothing selected, just create a new selection
+         */
+        if (!canvas->imageView()->selection()) {
+            action = SELECTION_REPLACE;
+        }
+
+        switch (action) {
+        case SELECTION_ADD:
+            actionName = kundo2_i18n("Select Opaque (Add)");
+            break;
+        case SELECTION_SUBTRACT:
+            actionName = kundo2_i18n("Select Opaque (Subtract)");
+            break;
+        case SELECTION_INTERSECT:
+            actionName = kundo2_i18n("Select Opaque (Intersect)");
+            break;
+        case SELECTION_SYMMETRICDIFFERENCE:
+            actionName = kundo2_i18n("Select Opaque (Symmetric Difference)");
+            break;
+        default:
+            actionName = kundo2_i18n("Select Opaque");
+            break;
+        }
+
+        qint32 x, y, w, h;
+        rc.getRect(&x, &y, &w, &h);
+
+        const KoColorSpace * cs = device->colorSpace();
+
+        KisHLineConstIteratorSP deviter = device->createHLineConstIteratorNG(x, y, w);
+        KisHLineIteratorSP selIter = tmpSel ->createHLineIteratorNG(x, y, w);
+
+        for (int row = y; row < h + y; ++row) {
+            do {
+                *selIter->rawData() = cs->opacityU8(deviter->oldRawData());
+            } while (deviter->nextPixel() && selIter->nextPixel());
+            deviter->nextRow();
+            selIter->nextRow();
+        }
+    }
+
+    KisSelectionToolHelper helper(canvas, actionName);
+    tmpSel->invalidateOutlineCache();
+    helper.selectPixelSelection(tmpSel, action);
+}
