@@ -38,7 +38,7 @@ public:
     KisFilterPosterize();
 public:
     KoColorTransformation* createTransformation(const KoColorSpace* cs, const KisFilterConfigurationSP config) const override;
-    KisConfigWidget* createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP dev) const override;
+    KisConfigWidget* createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP dev, bool useForMasks) const override;
     static inline KoID id() {
         return KoID("posterize", i18n("Posterize"));
     }
